//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Ax::Tools::String
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   07.09.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2006-03-03 15:49:48 +0100 (Fr, 03 Mär 2006) $
 *   $Revision: 1.3 $
 */
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
//----- C ---------------------------------------------------------------------
#include <stdio.h>
#include <stdarg.h>

//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>
#include <Ax/Tools/String.h>

//----- local -----------------------------------------------------------------

//=============================================================================
//     using namespaces
//=============================================================================

//=============================================================================
//     namespaces
//=============================================================================
namespace Ax
{
namespace Tools
{

//=============================================================================
//     class String
//=============================================================================

//---------------------------------------------------------------------
//     String::split()
//---------------------------------------------------------------------
Ax::Tools::StringVector String::split( const std::string &theSrc
                                     , const std::string &theSep
                                     , bool               fAllowEmpty
                                     )
{
  StringVector           aVec;
  std::string            aSrc = theSrc;
  std::string::size_type aPos = std::string::npos;
  std::string::size_type aSepLen = theSep.length();

  do
  {
    aPos = aSrc.find(theSep);

    std::string aSub = "";
    if (aPos == std::string::npos)
    {
      aSub = aSrc;
    }
    else
    {
      aSub = aSrc.substr(0, aPos);
      aSrc = aSrc.substr(aPos + aSepLen, aSrc.length() - aPos - aSepLen);
    } // if

    if (aSub.length() > 0 || fAllowEmpty)
    {
      aVec.push_back(aSub);
    } // if
  } while (aPos != std::string::npos);

  return aVec;
} // String::split()

//---------------------------------------------------------------------
//     String::split()
//---------------------------------------------------------------------
Ax::Tools::StringVector String::split( const std::string &theSrc
                                     , char               theSep
                                     , bool               fAllowEmpty
                                     )
{
  return split(theSrc, std::string(1, theSep), fAllowEmpty);
} // String::split()

//---------------------------------------------------------------------
//     String::concat()
//---------------------------------------------------------------------
std::string String::concat(const StringVector &theVec, std::string theFill)
{
  return concat(theVec.begin(), theVec.end(), theFill);
} // String::concat()

//---------------------------------------------------------------------
//     String::concat()
//---------------------------------------------------------------------
std::string String::concat(const StringVector &theVec, char theFill)
{
  return concat(theVec.begin(), theVec.end(), std::string(1, theFill));
} // String::concat()

//---------------------------------------------------------------------
//     String::concat()
//---------------------------------------------------------------------
std::string String::concat( StringVector::const_iterator theBegin
                               , StringVector::const_iterator theEnd
                               , std::string                  theFill
                               )
{
  std::string aStr;

  for (StringVector::const_iterator anIter = theBegin; anIter != theEnd; ++anIter)
  {
    if (aStr.length() > 0) aStr += theFill;
    aStr += *anIter;
  } // for

  return aStr;
} // String::concat()

//---------------------------------------------------------------------
//     String::concat()
//---------------------------------------------------------------------
std::string String::concat( StringVector::const_iterator theBegin
                          , StringVector::const_iterator theEnd
                          , char                         theFill
                          )
{
  return concat(theBegin, theEnd, std::string(1, theFill));
} // String::concat()

//---------------------------------------------------------------------
//     String::stripWhiteSpaces()
//---------------------------------------------------------------------
std::string String::stripWhiteSpaces(const std::string &theSrc)
{
  std::string aResult;
  std::string::size_type aPosFirst = theSrc.find_first_not_of(" \t");
  std::string::size_type aPosLast  = theSrc.find_last_not_of (" \t");

  if ( aPosFirst != std::string::npos
    && aPosLast  != std::string::npos
     )
  {
    aResult = theSrc.substr(aPosFirst, aPosLast - aPosFirst + 1);
  }
  else if (aPosLast != std::string::npos)
  {
    aResult = theSrc.substr(0, aPosLast + 1);
  }
  else if (aPosFirst != std::string::npos)
  {
    aResult = theSrc.substr(aPosFirst);
  }
  else
  {
    // both failed -> the string doesn't contain any readable characters
    aResult = "";
  } // if

  return aResult;
} // String::stripWhiteSpaces()

//---------------------------------------------------------------------
//     String::sprintf()
//---------------------------------------------------------------------
std::string String::sprintf(const char *theFmt, ...)
{
  va_list ap;
  va_start(ap, theFmt);
  char *buffer;
  vasprintf(&buffer, theFmt, ap);
  std::string aRet(buffer);
  free(buffer);
  return aRet;
} // String::sprintf()

}; // namespace Tools
}; // namespace Ax
