//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxMenuCode
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2006-05-14 10:37:24 +0200 (So, 14 Mai 2006) $
 *   $Revision: 1.1 $
 */
//-----------------------------------------------------------------------------

//=============================================================================
//     includes
//=============================================================================

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <algorithm>

//----- C ---------------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
#include <vdr/i18n.h>
#include <vdr/menuitems.h>

//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>

//----- local plugin ----------------------------------------------------------
#ifndef __AxVDROsdMenu_CPP__ // avoid including MenuCode.h -> AxOsdMenu.h -> AxOsdMenu.cpp which uses MenuCode
#define __AxVDROsdMenu_CPP__
#endif
#include <Ax/VDR/MenuCode.h>
#undef   __AxVDROsdMenu_CPP__

#include <Ax/VDR/Tools.h>
#include <Ax/VDR/Action.h>

//=============================================================================
//     using namespaces
//=============================================================================

//=============================================================================
//     namespaces
//=============================================================================
namespace Ax
{
namespace VDR
{

//----- locals ----------------------------------------------------------------

//-----------------------------------------------------------------------------
//     MenuCode::MenuCode()
//-----------------------------------------------------------------------------
MenuCode::MenuCode(Ax::VDR::Action *theAction)
        : PARENT( tr("MenuCode$Restricted area")
                , tr("MenuCode$Restricted area")
                , strlen(tr("MenuCode$Access code")) + 1
                )
        , myAction  (theAction)
        , myActCode (0)
        , myCodeItem(0)
{
  wsdebug(("> MenuCode::MenuCode() Code: '%s'", myAction->getLockCode().c_str()));

  //----- init attributes -------------------------------------------------
  myActCode = (char *)calloc(myAction->getLockCode().length() + 1, sizeof(char));

  //----- create some empty lines------------------------------------------
  for (int i = 1; i <= std::max(DisplayMenu()->MaxItems() / 2 - 1, 0); ++i)
  {
    cOsdItem *anItem = new cOsdItem("");
    anItem->SetSelectable(false);
    Add(anItem);
  } // for

  //----- create the access-code item -------------------------------------
  myCodeItem = new cMenuEditNumItem(tr("MenuCode$Access code"), myActCode, myAction->getLockCode().length(), true);
  Add(myCodeItem);
  SetCurrent(myCodeItem);

  //----- ensure no actions are on the numeric keys -----------------------
  setKeyAction(kOk  , A_NONE);
  setKeyAction(kBack, A_NONE);
  for (int aKey = k0; aKey <= k9; ++aKey)
  {
    setKeyAction(aKey, A_NONE);
   } // if

  wsdebug(("< MenuCode::MenuCode() Count: %d", Count()));
} // MenuCode::MenuCode()

//-----------------------------------------------------------------------------
//     MenuCode::~MenuCode()
//-----------------------------------------------------------------------------
MenuCode::~MenuCode()
{
  wsdebug(("> MenuCode::MenuCode()"));
  free(myActCode);
  wsdebug(("< MenuCode::~MenuCode()"));
} // MenuCode::~MenuCode()

#define DEBUG_PROCESS_KEY 0

//-------------------------------------------------------------------------
//     MenuCode::ProcessKey()
//-------------------------------------------------------------------------
eOSState MenuCode::ProcessKey(eKeys theKey)
{
#if DEBUG_PROCESS_KEY
  if (theKey != kNone)
  {
    wsdebug(("> MenuCode::ProcessKey()      Key: %-10s  Cur: %3d  HasSub: %d  parent processing..."
           , Ax::VDR::Tools::getKeyStr(theKey).c_str()
           , Current()
           , HasSubMenu()
           ));
  } // if
#endif

  eOSState aState = PARENT::ProcessKey(theKey);

#if DEBUG_PROCESS_KEY
  if (theKey != kNone)
  {
    wsdebug(("  MenuCode::ProcessKey()      Key: %-10s  Cur: %3d  HasSub: %d  State: %-10s  "
           , Ax::VDR::Tools::getKeyStr(theKey).c_str()
           , Current()
           , HasSubMenu()
           , Ax::VDR::Tools::getOSStateStr(aState)
           ));
  } // if
#endif

  //----- select action... ----------------------------------------------------
  switch (theKey)
  {
    case kBack:
    {
      aState = osBack;
      break;
    }

    //----- check the code and quit if ok or display error msg ----
    case kOk:
    {
      bool fCodeOk = strcmp(myActCode, myAction->getLockCode().c_str()) == 0;

      wsdebug(("  MenuCode::ProcessKey() CodeOK: %d  ReqCode: '%s'  ActCode: '%s'"
             , fCodeOk, myAction->getLockCode().c_str(), myActCode
             ));

      if (fCodeOk)
      {
        myAction->setLocked(false);
        aState = osBack;
      }
      else
      {
        memset(myActCode, 0, (myAction->getLockCode().length() + 1) * sizeof(char));
        showStatusMsg(tr("MenuCode$Code invalid!"));
        aState = osContinue;
      } // if
      break;
    }

    //----- check the code immediately and quit if ok -----
    case k0:
    case k1:
    case k2:
    case k3:
    case k4:
    case k5:
    case k6:
    case k7:
    case k8:
    case k9:
    {
      clearStatusMsg();
      // immediately close MenuCode if code is valid (without pressing the kOk)
      bool fCodeOk = strcmp(myActCode, myAction->getLockCode().c_str()) == 0;
      wsdebug(("  MenuCode::ProcessKey() CodeOK: %d  ReqCode: '%s'  ActCode: '%s'"
             , fCodeOk, myAction->getLockCode().c_str(), myActCode
             ));

      if (fCodeOk)
      {
        myAction->setLocked(false);
        aState = osBack;
      }
      myCodeItem->Set();
      Display();
      break;
    }

    default:
    {
#if 0
      aState = osContinue;
#endif
      break;
    }
  } // switch

#if DEBUG_PROCESS_KEY
  if (theKey != kNone)
  {
    wsdebug(("< MenuCode::ProcessKey()      Key: %-10s  Cur: %3d  HasSub: %d  State: %-10s  "
           , Ax::VDR::Tools::getKeyStr(theKey).c_str()
           , Current()
           , HasSubMenu()
           , Ax::VDR::Tools::getOSStateStr(aState)
           ));
  } // if
#endif
  return aState;
} // MenuCode::ProcessKey()

} // namespace VDR
} // namespace Ax
