//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxMailWrapper
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   28.06.2004
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2005-04-15 16:19:15 +0200 (Fr, 15 Apr 2005) $
 *   $Revision: 1.2 $
 */
//-----------------------------------------------------------------------------
#ifndef __AxMailWrapper_H__
#define __AxMailWrapper_H__

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <vector>

//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
#include <vdr/font.h>

//----- local -----------------------------------------------------------------

//----- other includes --------------------------------------------------------

//=============================================================================
//     forward declarations
//=============================================================================
namespace Ax
{
  namespace Mail
  {
    class Mail;
    class MailPart;
  };
};

//=============================================================================
//     class AxMailWrapper
//=============================================================================
class AxMailWrapper
{
  public:
    //-------------------------------------------------------------------------
    //     enums
    //-------------------------------------------------------------------------
    enum WrapMode
    {
      WM_FIRST   = 0

    , WM_NONE    = WM_FIRST   ///< let vdr do the word wrapping
    , WM_QUOTE   = 1          ///< continue quotes
    , WM_FLOWED  = 2          ///< honor 'format=flowed` (RFC 2646)
    , WM_SMALL   = 3          ///< continue lines which end and start with small chars

    , WM_COUNT   = 4
    };

  private:
    //-------------------------------------------------------------------------
    //     struct AxLine
    //-------------------------------------------------------------------------
    struct AxLine
    {
      //-----------------------------------------------------------------------
      AxLine(const std::string &theText = std::string(""), bool theFSep = false)
        : Text(theText)
        , fSep(theFSep)
      {
      }

      //-----------------------------------------------------------------------
      void clear()
      {
        Text = std::string("");
        fSep = false;
      }

      //-----------------------------------------------------------------------
      std::string Text;
      bool        fSep;
    }; // class AxLine

    //-------------------------------------------------------------------------
    //     typedefs
    //-------------------------------------------------------------------------
    typedef std::vector<AxLine> LineCltn;

  public:
    //-------------------------------------------------------------------------
    //     AxMailWrapper()
    //-------------------------------------------------------------------------
    AxMailWrapper(const cFont *theFont, int theWidth);

    //-------------------------------------------------------------------------
    //     ~AxMailWrapper()
    //-------------------------------------------------------------------------
    ~AxMailWrapper();

    //-------------------------------------------------------------------------
    //     getMailText()
    //-------------------------------------------------------------------------
    std::string getMailText(Ax::Mail::Mail *theMail, WrapMode theWrapMode = WM_NONE);

    //-------------------------------------------------------------------------
    //     wrapText()
    //-------------------------------------------------------------------------
    std::string wrapText(const std::string &theSrc, WrapMode theWrapMode = WM_NONE);

  private:
    //-----------------------------------------------------------------------------
    //     getQuotes()
    //-----------------------------------------------------------------------------
    std::string getQuotes(const std::string &theStr) const;

    //-------------------------------------------------------------------------
    //     cutLine()
    //-------------------------------------------------------------------------
    std::string cutLine(const std::string theSrc);

    //-------------------------------------------------------------------------
    //     splitText()
    //-------------------------------------------------------------------------
    LineCltn splitText(const std::string &theStr);

    //-------------------------------------------------------------------------
    //     getFlowedText()
    //-------------------------------------------------------------------------
    LineCltn getFlowedText(const LineCltn &theSrc);

    //-------------------------------------------------------------------------
    //     improveWrap()
    //-------------------------------------------------------------------------
    LineCltn improveWrap(const LineCltn &theSrc);

    //-------------------------------------------------------------------------
    //     wrapText()
    //-------------------------------------------------------------------------
    LineCltn wrapText(const LineCltn &theSrc);

    //-------------------------------------------------------------------------
    //     getPartText()
    //-------------------------------------------------------------------------
    LineCltn getPartText(const Ax::Mail::MailPart *thePart);

    //-------------------------------------------------------------------------
    //     dumpText()
    //-------------------------------------------------------------------------
    void dumpText(const std::string &theMsg, const LineCltn &theSrc);

    //-------------------------------------------------------------------------
    //     attributes
    //-------------------------------------------------------------------------
    const cFont *myFont ;
    int          myWidth;
    WrapMode     myWrapMode;
}; // class AxMailWrapper

#endif

